
// DetailsUsingIP.java
// Andrew Davison, November 2010, ad@fivedots.coe.psu.ac.th

/*  Contact SITE_ADDR to get details about this machine's IP address, 
    or a user-supplied address. The details 
    are stored in a HashMap, and some values printed out.

    Usage:
       > java DetailsUsingIP
    or
       > java DetailsUsingIP 66.12.74.247
*/

import java.util.*;
import java.io.*;
import java.net.*;


public class DetailsUsingIP 
{
  private static final String SITE_ADDR = 
                       "http://webipaddress.net/what-is-my-ip-address";


  public static void main (String[] args) 
  {
    String ipAddress = null;
    if (args.length == 0)
      System.out.println("Using own IP address...");
    else if (args.length == 1) {
      ipAddress = args[0];
      System.out.println("Using IP address: " + ipAddress + "...");
    }
    else
      System.out.println("Usage: java DetailsUsingIP [ <IP address> ]");

    String respStr = getResponse(ipAddress);
    showResponse(respStr);
  }  // end of main()



  private static String getResponse(String ipAddress) 
  /* Extracts the returned HTML table data between the 
     <div class="ipdetail"> and </DIV> tags.
     Example:

     <div class="ipdetail">
     <table width="380" cellpadding="0" cellspacing="6">
        <tr>
          <td valign="top"><strong>IP Information:</strong></td>
          <td width="223"><strong>202.12.74.4</strong></td>
        </tr>
        <tr>
          <td valign="top"><strong>ISP:</strong></td>
          <td>Prince of Songkla University</td>
          </tr>
        <tr>
          <td valign="top"><strong>Organization:</strong></td>
          <td>Prince of Songkla University</td>
          </tr>
        <tr>
          <td valign="top"><strong>City:</strong></td>
          <td>Songkhla</td>
          </tr>
        <tr>
          <td valign="top"><strong>Region / State:</strong></td>
          <td>68 Songkhla</td>
          </tr>
        <tr>
          <td valign="top"><strong>Country:</strong></td>
          <td>Thailand <img src='flag/th.gif' width='16' 
                height='11' title='Thailand' alt='TH'></img></td>
          </tr>
        <tr>
          <td valign="top"><strong>Country Code:</strong></td>
          <td>TH</td>
          </tr>
        <tr>
          <td width="115" valign="top"><strong>Continent:</strong></td>
          <td>AS</td>
          </tr>
        <tr>
          <td><strong>Time Zone:</strong></td>
          <td>Asia/Bangkok</td>
        </tr>
        <tr>
          <td><strong>Latitude:</strong></td>
          <td>7.1999998092651</td>
          </tr>
        <tr>
          <td><strong>Longitude:</strong></td>
          <td>100.59999847412</td>
          </tr>
        <tr>
          <td><strong>Postal Code:</strong></td>
          <td>n/a</td>
          </tr>
        <tr>
          <td><strong>Metro Code:</strong></td>
          <td>n/a</td>
          </tr>
        <tr>
          <td><strong>Area Code:</strong></td>
          <td>n/a</td>
          </tr>
      </table>
    </div>  
  */
  {
    try {
      URL url = null;
      if (ipAddress == null)
        url = new URL(SITE_ADDR);   // use own IP address
      else
        url = new URL(SITE_ADDR + "/ip/" + ipAddress);  // use supplied one

      URLConnection conn = url.openConnection();
      BufferedReader rd = new BufferedReader(
                        new InputStreamReader(conn.getInputStream()));

      boolean usefulInfo = false;
      StringBuilder tableStr = new StringBuilder();
      String line;
      while ((line = rd.readLine()) != null) {
         if (line.startsWith("<div class=\"ipdetail\">"))
           usefulInfo = true;
         if ((line.startsWith("</div>")) && usefulInfo)
           break;    // finish processing

         if (usefulInfo) {
           //System.out.println(line);
           tableStr.append(line.trim() + "\n");
         }
       }
       return tableStr.toString();
    } 
    catch (MalformedURLException e) 
    {  System.out.println(e); } 
    catch (IOException e) 
    {  System.out.println(e); } 

    return null;
   }  // end of getResponse()



  private static void showResponse(String respStr)
  /* print some of the response data after converting it to
     a HashMap of key:value pairs
  */
  {
    if (respStr == null)
      System.out.println("No address found");
    else {
      String tableData = extractData(respStr);
      HashMap<String,String> map = buildMap(tableData);
      if (map != null) {
        System.out.println("\nDetected IP address: " + 
                               map.get("IP Information") );
        System.out.println("(lat, long): (" + map.get("Latitude") +
                      ", " + map.get("Longitude") + ")");

        System.out.println("Address:");
        System.out.println("  " + map.get("Organization"));
        System.out.println("  " + map.get("City") + ", " +
                                  map.get("Region / State") );
        System.out.println("  " + map.get("Country") + ", " +
                                  map.get("Country Code") );
      }
    }
  }  // end of showResponse()


  private static String extractData(String respStr)
  /* Reformat HTML table data into key:value text lines.
     Example format:
           IP Information: 118.173.151.225
           ISP: TOT public company limited
           Organization: TOT Public Company Limited Bangkok
           City: Songkhla
           Region / State: 68 Songkhla
           Country: Thailand
           Country Code: TH
           Continent: AS
           Time Zone: Asia/Bangkok
           Latitude: 7.1999998092651
           Longitude: 100.59999847412
           Postal Code: n/a
           Metro Code: n/a
           Area Code: n/a
  */
  {
    // http://www.rgagnon.com/javadetails/java-0424.html
    String tableData = respStr.replaceAll("\\<.*?>","");  // remove tags

    tableData = tableData.replaceAll("(\\s){2}","$1");    
              // replace two consecutive white spaces with a single space

    tableData = tableData.replaceAll(":\\n", ": ");   // combine key and value lines

    tableData = tableData.replaceAll("\\n\\n", "\n").trim();  // remove blank lines

    // System.out.println("\nRetrieved Data:\n\n" + tableData);
    return tableData;
  }  // end of extractData()



  private static HashMap<String,String> buildMap(String tableData)
  /* convert the key:value text lines into a HashMap */
  {
    String[] lines = tableData.split("\\n");
    int numLines = lines.length;
    // System.out.println("\nFound " + numLines + " lines");

    if (numLines > 0) {
      HashMap<String,String> map = new HashMap<String,String>();
      String[] pair;
      for (int i=0; i < numLines; i++) {
        pair = lines[i].split(":");
        if (pair.length != 2)
          System.out.println("Problem parsing table line: \"" + lines[i] + "\"");
        else
          map.put(pair[0].trim(), pair[1].trim());   // key / value pairs
      }
      return map;
    }
    return null;
  }  // end of buildMap()


} // end of DetailsUsingIP class
