
// GoogleLatLong.java
// Andrew Davison, November 2010, ad@fivedots.coe.psu.ac.th

/* Use Google to lookup a geographical address for a given latitude
   and longitude. This is called reverse geocoding (see
   http://en.wikipedia.org/wiki/Reverse_geocoding).

  Reverse Geocoding in Google is described at:
    http://code.google.com/apis/maps/documentation/geocoding/#ReverseGeocoding

  There are some restrictions on its use:
    * a query limit of 2500 requests per day;

    * the Geocoding API may only be used in conjunction with a 
      Google map (so this program is a bit of a no-no :() 

  See OSMLatLong.java and YahooLatLong.java for other approaches.

  Usage:
    > java -cp "json.jar;." GoogleLatLong 7.0032811 100.4939958     // PSU
    > java  -cp "json.jar;." GoogleLatLong 53.314401 -2.255305      // UK

  Instead of -cp "json.jar;.", use compile and run. e.g.
    > compile GoogleLatLong.java
    > run GoogleLatLong 7.0032811 100.4939958
    > run GoogleLatLong 7.0073644 100.5012336

*/


import java.io.*;
import java.net.*;

import org.json.*;


public class GoogleLatLong 
{
  private static final String SITE_ADDR = 
                         "http://maps.google.com/maps/api/geocode/json";


  public static void main(String[] args) 
  {  
    if (args.length != 2) {
      System.out.println("Usage: run GoogleLatLong <latitude> <longitude>");
      return;
    }

    try {
      double lat = Double.parseDouble( args[0]);
      double lon = Double.parseDouble( args[1]);
      System.out.printf("Looking up (lat, long): (%.6f, %.6f)\n", lat, lon);

      String respStr = getResponse(lat, lon);
      showResponse(respStr); 
    }
    catch(NumberFormatException e)
    {  System.out.println("Incorrect format for latitude and longitude"); }
  }  // end of main()




  private static String getResponse(double lat, double lon) 
  /* Query Google, getting back address information in JSON format.
     Query format:
         http://maps.google.com/maps/api/geocode/json
             ?address=52.5487429714954,-1.81602098644987&
              sensor=false
  */
  {
    String coords = "" + lat + ", " + lon;
    BufferedReader rd = null;
    try {
      URL url = new URL(SITE_ADDR + "?address=" + 
                  URLEncoder.encode(coords, "UTF-8") + 
                     "&sensor=false");

      HttpURLConnection conn = (HttpURLConnection) url.openConnection();
      conn.connect();
      rd = new BufferedReader( new InputStreamReader(conn.getInputStream()));

      StringBuilder respStr = new StringBuilder();
      String line;
      while ((line = rd.readLine()) != null)
        respStr.append(line);
      rd.close();

      return respStr.toString(); 
    }
    catch (MalformedURLException e) 
    {  System.out.println("URL not understood"); } 
    catch (IOException e) 
    { System.out.println("I/O Error");  }

    return null;
  }  // end of getResponse()



  private static void showResponse(String respStr) 
  /* Store the JSON response in temp.json for future reference.
     Extract the formatted address information (there might be
     several), and print it out.
  */
  {  
    if (respStr == null)
      System.out.println("No response received");
    else {
      // System.out.println("Response:\n" + respStr);
      System.out.println("Processing response...");

      try {
        JSONObject json = new JSONObject(respStr);
        WebUtils.saveString("temp.json", json.toString(2) );    // indent the string

        String addr = extractAddress(json);
        if (addr != null)
          System.out.println("\nFormatted address: " + addr);
        else
          System.out.println("No address found");
      }
      catch(JSONException e)
      {  System.out.println(e);  }
    }
  }  // end of showResponse()



  private static String extractAddress(JSONObject json)
  /* Extract all the "formatted_address" strings from the JSON object. 
     Return the first one.
     JSON data format:
       {
         "results": [
           {
             "address_components": [ ... ],
             "formatted_address": "Thanon Punnakan, Hat Yai, Songkhla 90110, Thailand",
             "geometry": { ... },
             "types": ["route"]
           }, 
           ... ] }
  */
  {
    try {
      JSONArray jResults = json.getJSONArray("results");
      int numMatches = jResults.length();
      System.out.println("\nNo. of results: " + numMatches + "\n");

      if (numMatches == 0)
        return null;

      JSONObject jRes;
      for (int i = 0; i < numMatches; i++) {
        jRes = jResults.getJSONObject(i);
        System.out.println((i+1) + ". " + jRes.getString("formatted_address"));
      }
      return jResults.getJSONObject(0).getString("formatted_address");
    }
    catch(Exception e)
    {  System.out.println(e);  }

    return null;
  }  // end of extractAddress()


}  // end of GoogleLatLong class