
// GoogleMAC.java
// Andrew Davison, November 2010, ad@fivedots.coe.psu.ac.th

/* POST a query to http://www.google.com/loc/json supplying a MAC address
   and signal strength. Get back JSON data; extract and print the
   latitude, longitude, and geographical address.

   Some online info about this approach:
      http://www.e-rant.net/2010/06/google-wi-fi-hullabalooo/
      http://tinisles.blogspot.com/2009/12/wifi-geolocation.html

  Usage:
   java -cp "json.jar;." GoogleMAC 00:90:D0:f5:29:D2    // Hat Yai
   java -cp "json.jar;." GoogleMAC 00:19:D1:64:11:5B    // work  
   java -cp "json.jar;." GoogleMAC 0E:00:00:91:35:75    // London
   java -cp "json.jar;." GoogleMAC 00:02:2D:04:26:67    // Berlin

  Instead of -cp "json.jar;.", use compile and run. e.g.
    > compile GoogleMAC.java
    > run GoogleMAC 00:90:D0:f5:29:D2    // Hat Yai
    > run GoogleMAC 00:25:84:03:19:50    // PSU

*/

import java.util.*;
import java.io.*;
import java.net.*;

import org.json.*;


public class GoogleMAC
{
  private static final String SITE_ADDR = "http://www.google.com/loc/json";
  private static final int DUMMY_STRENGTH = -75;    
      // a strong signal, which means the AP is nearby


  public static void main(String[] args) 
  {
    String respStr = null;
    if (args.length == 1)
      respStr = getMACDetails(args[0], DUMMY_STRENGTH);
    else if (args.length == 2) {
      int signalStrength = Integer.parseInt(args[1]);
      respStr = getMACDetails(args[0], signalStrength);
    }
    else
      System.out.println("Usage: run GoogleMAC <MAC_address> [<signal strength>]");

    showResponse(respStr); 
  }  // end of main()



  private static String getMACDetails(String macAddress, int signalStrength)
  /* POST request format:
        {"version":"1.1.0",
         "request_address":true,
         "wifi_towers":[{"mac_address":"00:1C:DF:66:CA:F6",
                         "ssid":"",
                         "signal_strength":-30}]}
  */
  { 
    String data = "{\"version\":\"1.1.0\"," + 
                      "\"request_address\":true," +
                      "\"wifi_towers\": [{\"mac_address\":\"" + macAddress + 
                      "\",\"signal_strength\":" + signalStrength + "}]}";
    try {
      URL url = new URL(SITE_ADDR); 
      URLConnection conn = url.openConnection();
      conn.setDoInput(true); 
      conn.setDoOutput(true); 

      // write out JSON query parameters
      OutputStreamWriter writer = new OutputStreamWriter(conn.getOutputStream());
      writer.write(data);
      writer.flush();
      writer.close();

      // read the response
      BufferedReader in = new BufferedReader(new 
                                  InputStreamReader(conn.getInputStream()));
      StringBuilder respStr = new StringBuilder();
      String line;
      while ((line = in.readLine()) != null)
        respStr.append(line);
      in.close();

      return respStr.toString(); 
    } 
    catch (MalformedURLException e) 
    {  System.out.println("URL not understood"); } 
    catch (IOException e) 
    { System.out.println("I/O Error");  }

    return null;
  }   // end of getMACDetails()



  private static void showResponse(String respStr)
  /* Save the JSON response in "temp.json" for future reference.
     Print the extracted latitude, longitude, and geographical address.

    format of the response:
    {"location":{"latitude":51.4926781,"longitude":-0.2572147,"address":{"country":"
     United Kingdom","country_code":"GB","region":"Greater London","county":"Hounslow
     ","city":"Hounslow","street":"King St","street_number":"214","postal_code":"W4 2
     "},"accuracy":24.0},"access_token":"2:dofW0DNHaQPmu_f0:O64CNX4WzWxqdUif"}

    The address tuple may be absent.
  */
  {  
    if (respStr == null)
      System.out.println("No response received");
    else {
      // System.out.println("Response:\n" + respStr);
      try {
        JSONObject json = new JSONObject(respStr);
        WebUtils.saveString("temp.json", json.toString(2) );    // indent the string

        JSONObject jLoc = json.getJSONObject("location");
        showLatLong(jLoc);
        showAddress(jLoc);
      }
      catch(JSONException e)
      {  System.out.println(e);  }
    }
  }  // end of showResponse()



  private static void showLatLong(JSONObject jLoc)
  /* Extract the latitude and longitude from the JSON data.
     Data format:
       "latitude":51.4926781,"longitude":-0.2572147
       and perhaps ...,"accuracy":24.0}
  */
  {
    try {
      double lat = Double.parseDouble( getString(jLoc, "latitude"));
      double lon = Double.parseDouble( getString(jLoc, "longitude"));
      System.out.printf("(lat, long): (%.6f, %.6f)\n", lat, lon);

      String accuracy = getString(jLoc, "accuracy");
      if (accuracy != null)
        System.out.println("Accuracy: " + accuracy);
    }
    catch(NumberFormatException e)
    {  System.out.println("No latitude and longitude found"); }
  }  // end of showLatLong()



  private static String getString(JSONObject jo, String key)
  // return null if key lookup in JSONObject fails
  {
    try {
      return jo.getString(key);
    }
    catch(JSONException e){}
    return null;
  }  // end of getString()



  private static void showAddress(JSONObject jLoc)
  /* Extract the geographical address from the JSON data.
     Data format:
        "address":{"country":"
        United Kingdom","country_code":"GB","region":"Greater London","county":"Hounslow
        ","city":"Hounslow","street":"King St","street_number":"214","postal_code":"W4 2
        "}
     Print the address in a sensible format.
  */
  {
    JSONObject jAddr = null;
    try {
      jAddr = jLoc.getJSONObject("address");
    }
    catch(JSONException e){}

    if (jAddr == null)
      System.out.println("No address found");
    else {
      System.out.println("Address:");
      // 1st line
      System.out.print("  ");
      String streetNum = getString(jAddr, "street_number");
      if (streetNum != null)
        System.out.print(streetNum + " ");
      System.out.println(getString(jAddr, "street"));

      // 2nd line
      System.out.print("  ");
      String city = getString(jAddr, "city");
      if (city != null)
        System.out.print(city + ", ");

      String region = getString(jAddr, "region");
      if (region != null)
        System.out.print(region + ", ");
/*
      String county = getString(jAddr, "county");
      if (county != null)
        System.out.print(county + ", ");
*/
      System.out.println( getString(jAddr, "postal_code"));

      // 3rd line
      System.out.print("  ");
      String country = getString(jAddr, "country");
      if (country != null)
        System.out.print(country + ", ");

      System.out.println( getString(jAddr, "country_code"));
    }
  }  // end of showAddress()

}  // end of GoogleMAC class
