
// OSMLatLong.java
// Andrew Davison, November 2010, ad@fivedots.coe.psu.ac.th

/* Use OpenStreetMap (OSM) to lookup a geographical address 
   for a given latitude and longitude. This is called reverse 
   geocoding (see http://en.wikipedia.org/wiki/Reverse_geocoding).

  Reverse Geocoding in OSM is described at:
    http://wiki.openstreetmap.org/wiki/Nominatim

  See GoogleLatLong.java and YahooLatLong.java for other approaches.

  Usage:
    > java -cp "json.jar;." OSMLatLong 7.0032811 100.4939958      // PSU
    > java  -cp "json.jar;." OSMLatLong  53.314401 -2.255305      //UK

  Instead of -cp "json.jar;.", use compile and run. e.g.
    > compile OSMLatLong.java
    > run OSMLatLong 7.0032811 100.4939958
*/


import java.io.*;
import java.net.*;

import org.json.*;


public class OSMLatLong 
{
  private static final String SITE_ADDR = 
                  "http://nominatim.openstreetmap.org/reverse";


  public static void main(String[] args) 
  {  
    if (args.length != 2) {
      System.out.println("Usage: run OSMLatLong <latitude> <longitude>");
      return;
    }

    try {
      double lat = Double.parseDouble( args[0]);
      double lon = Double.parseDouble( args[1]);
      System.out.printf("Looking up (lat, long): (%.6f, %.6f)\n", lat, lon);

      String respStr = getResponse(lat, lon);
      showResponse(respStr); 
    }
    catch(NumberFormatException e)
    {  System.out.println("Incorrect format for latitude and longitude"); }
  }  // end of main()



  private static String getResponse(double lat, double lon) 
  /* Query OSM, getting back address information in JSON format
     Query format:
         http://nominatim.openstreetmap.org/reverse
             ?format=json&lat=7.0073644&lon=100.5012336&
                        zoom=18&addressdetails=1
  */
  {
    BufferedReader rd = null;
    try {
      URL url = new URL(SITE_ADDR + "?format=json&" + 
                     "lat=" + lat + "&lon=" + lon + "&" +
                     "zoom=18&addressdetails=1");

      HttpURLConnection conn = (HttpURLConnection) url.openConnection();
      conn.connect();
      rd = new BufferedReader(
         new InputStreamReader(conn.getInputStream()));

      StringBuilder respStr = new StringBuilder();
      String line;
      while ((line = rd.readLine()) != null)
        respStr.append(line);
      rd.close();

      return respStr.toString(); 
    }
    catch (MalformedURLException e) 
    {  System.out.println("URL not understood"); } 
    catch (IOException e) 
    { System.out.println("I/O Error");  }

    return null;
  }  // end of getResponse()



  private static void showResponse(String respStr)
  /* Store the JSON response in temp.json for future reference.
     Extract the "display_name" information, and print it out.
     JSON data format:
       {
         "address": {
           "country": "United Kingdom",
           "country_code": "gb",
           "county": "Cheshire East",
           "hamlet": "Chorley",
           "house_number": "15",
           "postcode": "SK9 6EQ",
           "road": "Davenport Avenue",
           "state": "England",
           "state_district": "North West England"
         },
         "category": "place",
         "display_name": "15, Davenport Avenue, Chorley, Cheshire East, ...",
         "licence": "Data Copyright OpenStreetMap Contributors, ...",
         "osm_id": "530060459",
         "osm_type": "node",
         "place_id": "118639120",
         "type": "house"
       }

      Some fields may be missing.
  */
  {  
    if (respStr == null)
      System.out.println("No response received");
    else {
      // System.out.println("Response:\n" + respStr);
      try {
        JSONObject json = new JSONObject(respStr);
        WebUtils.saveString("temp.json", json.toString(2) );    // indent the string

        System.out.println("Street Address: " + json.getString("display_name"));
      }
      catch(JSONException e)
      {  System.out.println(e);  }
    }
  }  // end of showResponse()


}  // end of OSMLatLong class