
// SkyhookMAC.java
// Andrew Davison, November 2010, ad@fivedots.coe.psu.ac.th

/* 
  Access Skyhook location serviice at
     http://www.skyhookwireless.com/
  Send it a HTTPS POST request containing XML in the 
  xmlQuery string format. Get back latitude and longitude information.

  Based on the blog post:
     http://coderrr.wordpress.com/2008/09/10/
           get-the-physical-location-of-wireless-router-from-its-mac-address-bssid/
  and
     http://www.attackvector.org/geolocation-using-bssid/

  Don't forget to assign values to the USER_NAME and REALM constants 
  at SDK registration time.

  Usage:
   java SkyhookMAC <MAC address> [ <signal stength> ]
  e.g.
   java SkyhookMAC 00:90:d0:f5:29:d2     // oak tree

  On success:

    <?xml version="1.0" encoding="UTF-8" standalone="yes"?>
    <LocationRS version="2.10" xmlns="http://skyhookwireless.com/wps/2005">
      <location nap="1">
        <latitude>7.199997</latitude>
        <longitude>100.600006</longitude>
        <hpe>150</hpe>
      </location>
    </LocationRS>

    There may also be address information; see
    http://www.attackvector.org/geolocation-using-bssid/ for some details

  On failure (which I always get):

    <?xml version="1.0" encoding="UTF-8" standalone="yes"?>
    <LocationRS version="2.10" xmlns="http://skyhookwireless.com/wps/2005">
       <error>Unable to determine location</error>
    </LocationRS>

  I believe this is due to my location in Thailand being outside
  Skyhook's coverage. 
*/

import java.util.*;
import java.io.*;
import java.net.*;
import javax.net.ssl.*;

import javax.xml.parsers.*;
import javax.xml.xpath.*;
import org.w3c.dom.*;
import org.xml.sax.*;


public class SkyhookMAC
{
  private static final String SITE_ADDR = 
                      "https://api.skyhookwireless.com/wps2/location";
  private static final int DUMMY_STRENGTH = -75;    
      // a strong signal, which means the AP is nearby

  // **** CHANGE THE FOLLOWING ACCORDING TO YOUR SKYHOOK REGISTRATION ****
  private static final String USER_NAME = "????";
  private static final String REALM = "????";



  public static void main(String[] args) 
  {  
    String macAddress = null;
    int signalStrength = DUMMY_STRENGTH; 
    if (args.length == 1)
      macAddress = args[0];
    else if (args.length == 2) {
      macAddress = args[0];
      signalStrength = Integer.parseInt(args[1]);
    }
    else {
      System.out.println("Usage: java SkyhookMAC <MAC address> [<signal>]");
      return;
    }

    String xmlQuery = "<?xml version='1.0'?>\n" +
         "<LocationRQ xmlns='http://skyhookwireless.com/wps/2005'" +
              " version='2.10' street-address-lookup='full'>\n" +
              "<authentication version='2.0'>\n" +
                "<simple>\n" +
                  "<username>" + USER_NAME + "</username>\n" +
                  "<realm>" + REALM + "</realm>  " +
                "</simple>\n" +
              "</authentication>  \n" +
              "<access-point>  \n" +
                "<mac>" +  macAddress + "</mac>  \n" +
                "<signal-strength>" + signalStrength + "</signal-strength>\n" +
                "<age>7984</age>\n" +
              "</access-point>  \n" +
         "</LocationRQ>";

    // System.out.println("Generated XML:\n" + xmlQuery);
    String respStr = getResponse(xmlQuery);
    if (respStr == null)
      System.out.println("No response received");
    else {
      // System.out.println("Response:\n" + respStr);
      extractDetails(respStr);
    }
  }  // end of main()


  private static String getResponse(String xmlQuery)
  { 
    try {
      URL url = new URL(SITE_ADDR); 
      // open SSL https connection 
      HttpsURLConnection conn = (HttpsURLConnection) url.openConnection(); 

      conn.setDoInput(true); 
      conn.setDoOutput(true); 
      conn.setRequestProperty("Content-Type", "text/xml"); // send XML info

      // POST the XML query
      PrintWriter output = new PrintWriter(
                      new OutputStreamWriter(conn.getOutputStream())); 
      output.println(xmlQuery); 
      output.flush();
      output.close(); 
      conn.connect(); 

      // read the response
      BufferedReader in = new BufferedReader(new 
                              InputStreamReader(conn.getInputStream()));
      StringBuilder respStr = new StringBuilder();
      String line;
      while ((line = in.readLine()) != null)
        respStr.append(line);
      in.close();
      return respStr.toString(); 
    } 
    catch (MalformedURLException e) 
    {  System.out.println("URL not understood"); } 
    catch (IOException e) 
    { System.out.println("I/O Error");  }

    return null;
  }   // end of getResponse()



  private static void extractDetails(String respStr)
  // extract the latitude and longitude, or report the error;
  // I've only been able to test the "/LocationRS/error" code branch
  {
    try {
      InputSource is = new InputSource(new StringReader(respStr));
      Document respDoc = DocumentBuilderFactory.newInstance().
                                    newDocumentBuilder().parse(is);
      XPath xPath = XPathFactory.newInstance().newXPath();

      // Get the Location nodes using XPath
      NodeList nodes = (NodeList)xPath.evaluate("/LocationRS/Location", 
                                respDoc, XPathConstants.NODESET);
      int nodeCount = nodes.getLength();
      if (nodeCount != 0) {
        System.out.println("No. of location nodes: " + nodeCount);
        // only report the first latitude and longitude
        String lat = (String)xPath.evaluate("latitude", 
                                  nodes.item(0), XPathConstants.STRING);
        String lon = (String)xPath.evaluate("longitude", 
                                  nodes.item(0), XPathConstants.STRING);
        if (lat == null)
          System.out.println("No latitude and longitude found");
        else
          System.out.println("First (lat,lon): (" + lat + ", " + lon + ")");
      }
      else {
        String errorMsg = (String)xPath.evaluate("/LocationRS/error", 
                                  respDoc, XPathConstants.STRING);
        if (errorMsg != null)
          System.out.println("Skyhook error: " + errorMsg);
        else
          System.out.println("Could not parse the response string");
      }
    }
    catch(Exception e)
    {  System.out.println(e); }
  }  // end of extractDetails()


}  // end of SkyhookMAC class
