
// Whois.java
// Andrew Davison, November 2010, ad@fivedots.coe.psu.ac.th

/* POST a WHOIS query to http://uwhois.com/ including an IP
   address to lookup. Multiple Whois records may be returned
   in the web page response. Extract some of the information
   and print it out

  Usage:
     > java Whois 202.12.74.244
*/

import java.util.*;
import java.io.*;
import java.net.*;


public class Whois
{

  private static final String SITE_ADDR = "http://uwhois.com/cgi/whois.cgi";

  // lists for holding whois data 
  // on descriptions, addresses, and countries
  private static ArrayList<String> descs, addrs, cntrs;


  public static void main(String[] args) 
  {
    if (args.length != 1)
      System.out.println("Usage: java Whois <IP_address>");
    else {
      descs = new ArrayList<String>();
      addrs = new ArrayList<String>();
      cntrs = new ArrayList<String>();
      String respStr = queryWhois(args[0]);
      showResponse(respStr); 
    }
  }  // end of main()



  private static String queryWhois(String ipAddress)
  /* POST request format:
        query=<IP address>
  */
  { 
    System.out.println("Querying whois for " + ipAddress + "...");
    try {
      URL url = new URL(SITE_ADDR); 
      URLConnection conn = url.openConnection();
      conn.setDoInput(true); 
      conn.setDoOutput(true); 

      // post the query
      OutputStreamWriter writer = new OutputStreamWriter(conn.getOutputStream());
      writer.write("query="+ ipAddress);
      writer.flush();
      writer.close();

      // read the response
      BufferedReader in = new BufferedReader(new 
                                  InputStreamReader(conn.getInputStream()));
      StringBuilder respStr = new StringBuilder();
      String line;
      while ((line = in.readLine()) != null)
        respStr.append(line + "\n");    // retain newline so easy to extract data
      in.close();

      return respStr.toString(); 
    } 
    catch (MalformedURLException e) 
    {  System.out.println("URL not understood"); } 
    catch (IOException e) 
    { System.out.println("I/O Error");  }

    return null;
  }   // end of queryWhois()



  private static void showResponse(String respStr)
  /* Strip away the web page's tags, and then look for lines of the form:
        person:         Pansak Arpakajorn
        e-mail:         abuse@totisp.net
        address:        TOT Public Company Limited
        address:        89/2 Moo 3 Chaengwattana Rd, Laksi,Bangkok 10210 THAILAND
        phone:          +66-2574-9178
    Collect the "descr", "address", "country" values in lists,
    and then print them out in order at the end of the page's processing.
  */
  {  
    if (respStr == null)
      System.out.println("No response received");
    else {
      respStr = respStr.replaceAll("\\<.*?>","");  // remove tags
      // System.out.println("Response:\n" + respStr);   // print raw text

      System.out.println("Response:");
      int numLines = 0;
      String[] pairs;   // for key/value pairs
      String[] lines = respStr.split("\\n");
      for (String line : lines) {
         if ( line.startsWith("descr:") || line.startsWith("address:") ||
              line.startsWith("country:") ) {
           // System.out.println(line);
           pairs = line.split(":");
           addToLists(pairs[0].trim(), pairs[1].trim());   // key,value
           numLines++;
         }
      }

      if (numLines == 0)
        System.out.println("  Nothing suitable found");
      else {
        printList(descs);
        printList(addrs);
        if (cntrs.size() > 0)
          System.out.println(cntrs.get(0));
      }
    }
  }  // end of showResponse()



  private static void addToLists(String key, String value)
  // store the values for the "descr", "address", "country" keys
  {
    if (key.equals("descr"))
      descs.add(value);
    else if (key.equals("address"))
      addrs.add(value);
    else if (key.equals("country"))
      cntrs.add(value);
    else
      System.out.println("Did not recognise key: \"" + key + "\"");
  }  // end of addToLists()



  private static void printList(ArrayList<String> details)
  // print first value on a line, then others indented on their own lines
  {
    if (details.size() > 0) {
      System.out.println(details.get(0));
      for(int i=1; i < details.size(); i++)
        System.out.println(" - " + details.get(i));
    }
  }  // end of printList()

}  // end of Whois class
