
// YahooLatLong.java
// Andrew Davison, November 2010, ad@fivedots.coe.psu.ac.th

/* Use Yahoo! to lookup a geographical address for a given latitude
   and longitude. This is called reverse geocoding (see
   http://en.wikipedia.org/wiki/Reverse_geocoding).

  Reverse Geocoding in Yahoo! is described at:
         http://developer.yahoo.com/geo/placefinder/
  and in a user guide:
    http://developer.yahoo.com/geo/placefinder/guide

  Yahoo! PlaceFinder developer forum:
    http://developer.yahoo.net/forum/?showforum=124&cookiecheckonly=1

  There are some restrictions on its use:
    * you must sign up for an application ID;
    * a query limit of 50,000 requests per day

  Don't forget to sign up for a Yahoo application ID for the APP_ID constant.

  See GoogleLatLong.java and OSMLatLong.java for other approaches.

  Usage:
    > java -cp "json.jar;." YahooLatLong 7.0032811 100.4939958     // PSU
    > java -cp "json.jar;." YahooLatLong 53.314401 -2.255305       // UK

  Instead of -cp "json.jar;.", use compile and run. e.g.
    > compile YahooLatLong.java
    > run YahooLatLong 7.0032811 100.4939958

*/

import java.io.*;
import java.net.*;
import org.json.*;


public class YahooLatLong 
{
  private static final String SITE_ADDR = "http://where.yahooapis.com/geocode";

  // **** FILL THIS IN WITH YOUR YAHOO APPLICATION ID *****
  private static final String APP_ID = "????";



  public static void main(String[] args) 
  {  
    if (args.length != 2) {
      System.out.println("Usage: run YahooLatLong <latitude> <longitude?");
      return;
    }

    try {
      double lat = Double.parseDouble( args[0]);
      double lon = Double.parseDouble( args[1]);
      System.out.printf("Looking up (lat, long): (%.6f, %.6f)\n", lat, lon);

      String respStr = getResponse(lat, lon);
      showResponse(respStr); 
    }
    catch(NumberFormatException e)
    {  System.out.println("Incorrect format for latitude and longitude"); }
  }  // end of main()



  private static String getResponse(double lat, double lon) 
  /* Query Yahoo!, getting back address information in JSON format.
     Query format:
         http://where.yahooapis.com/geocode
            ?location=38.898717+-77.035974&gflags=R&flags=J&
            appid=[yourappidhere]
  */
  {
    String coords = "?location=" + lat + ", " + lon + "&";
    BufferedReader rd = null;
    try {
      URL url = new URL(SITE_ADDR + 
                    "?location=" + lat + "+" + lon + "&" +
                     "gflags=R&flags=J&" +
                     "appid=" + APP_ID);

      HttpURLConnection conn = (HttpURLConnection) url.openConnection();
      conn.connect();
      rd = new BufferedReader(
         new InputStreamReader(conn.getInputStream()));

      StringBuilder respStr = new StringBuilder();
      String line;
      while ((line = rd.readLine()) != null)
        respStr.append(line);
      rd.close();

      return respStr.toString(); 
    }
    catch (MalformedURLException e) 
    {  System.out.println("URL not understood"); } 
    catch (IOException e) 
    { System.out.println("I/O Error");  }

    return null;
  }  // end of getResponse()




  private static void showResponse(String respStr)
  /* Store the JSON response in temp.json for future reference.
     Access the Results array in the ResultSet tuple.
     JSON data format:
          {"ResultSet": {
              "Error": 0,
              "ErrorMessage": "No error",
              "Found": 1,
              "Locale": "us_US",
              "Quality": 99,
              "Results": [{ ...   }],
              "version": "1.0"
          }}
     Some fields may be missing.
  */
  {  
    if (respStr == null)
      System.out.println("No response received");
    else {
      // System.out.println("Response:\n" + respStr);
      try {
        JSONObject json = new JSONObject(respStr);
        WebUtils.saveString("temp.json", json.toString(2) );    // indent the string

        JSONObject jResSet = json.getJSONObject("ResultSet");
        showAddress(jResSet);
      }
      catch(JSONException e)
      {  System.out.println(e);  }
    }
  }  // end of showResponse()


  private static String getString(JSONObject jo, String key)
  // return null if a key is missing from the JSONObject
  {
    try {
      return jo.getString(key);
    }
    catch(JSONException e){}
    return null;
  }  // end of getString()



  private static void showAddress(JSONObject jResSet)
  /* Extract the address details from the first Results tuple.
     The details are constructed from the "line fields, which 
     are printed out over 4 lines. 
     JSON data format:
          "Results": [{
              "city": "Wilmslow",
              "country": "United Kingdom",
              "countrycode": "GB",
              "county": "Cheshire",
              "countycode": "CHS",
              "hash": "",
              "house": "21",
              "latitude": "53.314401",
              "line1": "21 Davenport Avenue",
              "line2": "Wilmslow",
              "line3": "SK9 6",
              "line4": "United Kingdom",
              "longitude": "-2.255305",
              "name": "53.314401 -2.255305",
              "neighborhood": "",
              "offsetlat": "53.314401",
              "offsetlon": "-2.255305",
              "postal": "SK9 6",
              "quality": 99,
              "radius": 500,
              "state": "England",
              "statecode": "ENG",
              "street": "Davenport Avenue",
              "unit": "",
              "unittype": "",
              "uzip": "SK9 6",
              "woeid": 26351559,
              "woetype": 11,
              "xstreet": ""
            }],
  */
  {
    JSONObject jRes = null;
    try {
      JSONArray jResults = jResSet.getJSONArray("Results");
      int numResults = jResults.length();
      if (numResults > 1)
        System.out.println("No. of results: " + numResults + "\n");
      if (numResults >= 1)
        jRes = jResults.getJSONObject(0);
    }
    catch(JSONException e){}

    if (jRes == null)
      System.out.println("No address found");
    else {
      System.out.println("Address:");
      System.out.println("  " + getString(jRes, "line1"));
      System.out.println("  " + getString(jRes, "line2"));
      System.out.println("  " + getString(jRes, "line3"));
      System.out.println("  " + getString(jRes, "line4"));
    }
  }  // end of showAddress()


}  // end of YahooLatLong class